Class: Datadog::Core::Configuration::Components

Inherits:
Object
  • Object
show all
Extended by:
Tracing::Component
Includes:
Tracing::Component::InstanceMethods
Defined in:
lib/datadog/core/configuration/components.rb

Overview

Global components for the trace library.

Constant Summary

Constants included from Tracing::Component

Tracing::Component::WRITER_RECORD_ENVIRONMENT_INFORMATION_CALLBACK

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Tracing::Component

build_sampler, build_span_sampler, build_trace_flush, build_tracer, build_writer, ensure_priority_sampling, subscribe_to_writer_events!, writer_update_priority_sampler_rates_callback

Methods included from Tracing::Component::InstanceMethods

#reconfigure_live_sampler

Constructor Details

#initialize(settings) ⇒ Components

Returns a new instance of Components.



82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
# File 'lib/datadog/core/configuration/components.rb', line 82

def initialize(settings)
  @logger = self.class.build_logger(settings)

  # This agent_settings is intended for use within Core. If you require
  # agent_settings within a product outside of core you should extend
  # the Core resolver from within your product/component's namespace.
  agent_settings = AgentSettingsResolver.call(settings, logger: @logger)

  @remote = Remote::Component.build(settings, agent_settings)
  @tracer = self.class.build_tracer(settings, logger: @logger)

  @profiler = Datadog::Profiling::Component.build_profiler_component(
    settings: settings,
    agent_settings: agent_settings,
    optional_tracer: @tracer,
  )
  @runtime_metrics = self.class.build_runtime_metrics_worker(settings)
  @health_metrics = self.class.build_health_metrics(settings)
  @telemetry = self.class.build_telemetry(settings, agent_settings, logger)
  @appsec = Datadog::AppSec::Component.build_appsec_component(settings)
end

Instance Attribute Details

#appsecObject (readonly)

Returns the value of attribute appsec.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def appsec
  @appsec
end

#health_metricsObject (readonly)

Returns the value of attribute health_metrics.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def health_metrics
  @health_metrics
end

#loggerObject (readonly)

Returns the value of attribute logger.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def logger
  @logger
end

#profilerObject (readonly)

Returns the value of attribute profiler.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def profiler
  @profiler
end

#remoteObject (readonly)

Returns the value of attribute remote.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def remote
  @remote
end

#runtime_metricsObject (readonly)

Returns the value of attribute runtime_metrics.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def runtime_metrics
  @runtime_metrics
end

#telemetryObject (readonly)

Returns the value of attribute telemetry.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def telemetry
  @telemetry
end

#tracerObject (readonly)

Returns the value of attribute tracer.



72
73
74
# File 'lib/datadog/core/configuration/components.rb', line 72

def tracer
  @tracer
end

Class Method Details

.build_health_metrics(settings) ⇒ Object



23
24
25
26
27
28
29
# File 'lib/datadog/core/configuration/components.rb', line 23

def build_health_metrics(settings)
  settings = settings.diagnostics.health_metrics
  options = { enabled: settings.enabled }
  options[:statsd] = settings.statsd unless settings.statsd.nil?

  Core::Diagnostics::Health::Metrics.new(**options)
end

.build_logger(settings) ⇒ Object



31
32
33
34
35
36
# File 'lib/datadog/core/configuration/components.rb', line 31

def build_logger(settings)
  logger = settings.logger.instance || Core::Logger.new($stdout)
  logger.level = settings.diagnostics.debug ? ::Logger::DEBUG : settings.logger.level

  logger
end

.build_runtime_metrics(settings) ⇒ Object



38
39
40
41
42
43
44
# File 'lib/datadog/core/configuration/components.rb', line 38

def build_runtime_metrics(settings)
  options = { enabled: settings.runtime_metrics.enabled }
  options[:statsd] = settings.runtime_metrics.statsd unless settings.runtime_metrics.statsd.nil?
  options[:services] = [settings.service] unless settings.service.nil?

  Core::Runtime::Metrics.new(**options)
end

.build_runtime_metrics_worker(settings) ⇒ Object



46
47
48
49
50
51
52
53
54
# File 'lib/datadog/core/configuration/components.rb', line 46

def build_runtime_metrics_worker(settings)
  # NOTE: Should we just ignore building the worker if its not enabled?
  options = settings.runtime_metrics.opts.merge(
    enabled: settings.runtime_metrics.enabled,
    metrics: build_runtime_metrics(settings)
  )

  Core::Workers::RuntimeMetrics.new(options)
end

.build_telemetry(settings, agent_settings, logger) ⇒ Object



56
57
58
59
60
61
62
63
64
65
66
67
# File 'lib/datadog/core/configuration/components.rb', line 56

def build_telemetry(settings, agent_settings, logger)
  enabled = settings.telemetry.enabled
  if agent_settings.adapter != Datadog::Core::Configuration::Ext::Agent::HTTP::ADAPTER
    enabled = false
    logger.debug { "Telemetry disabled. Agent network adapter not supported: #{agent_settings.adapter}" }
  end

  Telemetry::Client.new(
    enabled: enabled,
    heartbeat_interval_seconds: settings.telemetry.heartbeat_interval_seconds
  )
end

Instance Method Details

#shutdown!(replacement = nil) ⇒ Object

Shuts down all the components in use. If it has another instance to compare to, it will compare and avoid tearing down parts still in use.



125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
# File 'lib/datadog/core/configuration/components.rb', line 125

def shutdown!(replacement = nil)
  # Shutdown remote configuration
  remote.shutdown! if remote

  # Decommission AppSec
  appsec.shutdown! if appsec

  # Shutdown the old tracer, unless it's still being used.
  # (e.g. a custom tracer instance passed in.)
  tracer.shutdown! unless replacement && tracer == replacement.tracer

  # Shutdown old profiler
  profiler.shutdown! unless profiler.nil?

  # Shutdown workers
  runtime_metrics.stop(true, close_metrics: false)

  # Shutdown the old metrics, unless they are still being used.
  # (e.g. custom Statsd instances.)
  #
  # TODO: This violates the encapsulation created by Runtime::Metrics and
  # Health::Metrics, by directly manipulating `statsd` and changing
  # it's lifecycle management.
  # If we need to directly have ownership of `statsd` lifecycle, we should
  # have direct ownership of it.
  old_statsd = [
    runtime_metrics.metrics.statsd,
    health_metrics.statsd
  ].compact.uniq

  new_statsd =  if replacement
                  [
                    replacement.runtime_metrics.metrics.statsd,
                    replacement.health_metrics.statsd
                  ].compact.uniq
                else
                  []
                end

  unused_statsd = (old_statsd - (old_statsd & new_statsd))
  unused_statsd.each(&:close)

  telemetry.stop!
  telemetry.emit_closing! unless replacement
end

#startup!(settings) ⇒ Object

Starts up components



105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
# File 'lib/datadog/core/configuration/components.rb', line 105

def startup!(settings)
  if settings.profiling.enabled
    if profiler
      @logger.debug('Profiling started')
      profiler.start
    else
      # Display a warning for users who expected profiling to be enabled
      unsupported_reason = Profiling.unsupported_reason
      logger.warn("Profiling was requested but is not supported, profiling disabled: #{unsupported_reason}")
    end
  else
    @logger.debug('Profiling is disabled')
  end

  Core::Diagnostics::EnvironmentLogger.collect_and_log!
end