Module: Datadog::Core::Encoding::Encoder Abstract
Overview
This module is abstract.
Encoder interface that provides the logic to encode traces and service
Instance Method Summary collapse
- #content_type ⇒ Object
-
#encode(_) ⇒ Object
Serializes a single trace into a String suitable for network transmission.
-
#join(encoded_elements) ⇒ Object
Concatenates a list of elements previously encoded by
#encode.
Instance Method Details
#content_type ⇒ Object
15 16 17 |
# File 'lib/datadog/core/encoding.rb', line 15 def content_type raise NotImplementedError end |
#encode(_) ⇒ Object
Serializes a single trace into a String suitable for network transmission.
25 26 27 |
# File 'lib/datadog/core/encoding.rb', line 25 def encode(_) raise NotImplementedError end |
#join(encoded_elements) ⇒ Object
Concatenates a list of elements previously encoded by #encode.
20 21 22 |
# File 'lib/datadog/core/encoding.rb', line 20 def join(encoded_elements) raise NotImplementedError end |